
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: philip a. wilsey
--phil.wilsey@uc.edu


use std.textio.all;
package body compose is


--some conversion utilities to make life easier

function stringof (in_number : integer) return string is

variable line, reverse_line : string (1 to 10) := "          ";
variable  i, j, wk_number : integer;
type char_array is array (integer range 0 to 9) of character;
variable digit_to_letter : char_array := ('0', '1', '2', '3', '4',
                                          '5', '6', '7', '8', '9');

begin

    if in_number = 0 then 
    	return "0";
    end if;
    i := 1;
    wk_number := abs in_number;
    while wk_number > 0 loop
    	reverse_line(i) := digit_to_letter(wk_number mod 10);
	wk_number := wk_number / 10;
	i := i + 1;
    end loop;

    j := 1;
    while j < i loop
    	line(j + 1) := reverse_line(i - j);
	j := j + 1;
    end loop;
    
    if (in_number >= 0) then
    	return line(2 to j);
    else
    	line(1) := '-';
    	return line(1 to j);
    end if;

end stringof;

function stringof (in_bv : bit_vector) return string is

variable line : string (in_bv'range);
type char_array is array (bit) of character;
variable bit_to_char : char_array := ('0', '1');


begin

    for i in in_bv'range loop
    	line(i) := bit_to_char(in_bv(i));
    end loop;
    return line;

end stringof;

function stringof (in_b : bit) return string is

variable line : string (1 to 1);
type char_array is array (bit) of character;
variable bit_to_char : char_array := ('0', '1');


begin

    line(1) := bit_to_char(in_b);
    return line;

end stringof;

function stringof (wk_time : time) return string is

begin

    return stringof(wk_time / 1 fs) & " fs";  --convert to integer and
					      --then use integer fn

end stringof;

function intof (in_bv : bit_vector) return integer is

variable wk_integer : integer;
type integer_array is array (bit) of integer;
variable bit_to_integer : integer_array := (0, 1);

begin

    assert in_bv'length <= 32
    	report "Size of bit vector exceeds integer width.  Conversion aborted."
	severity failure;

    if (in_bv(in_bv'left) = '1') then
        return -(intof(not in_bv)+1);
    else
        wk_integer := 0;
	for i in in_bv'range loop
	    wk_integer := (wk_integer * 2) + bit_to_integer(in_bv(i));
	end loop;
    end if;
    return wk_integer;

end intof;

function bitvectorof (in_number : integer) return bit_vector is

variable wk_bv : bit_vector (31 downto 0);
variable wk_number, i : integer;
type bit_array is array (bit) of bit;
variable invert_bit : bit_array := ('1', '0');

begin

    wk_number := abs in_number;
    for i in 0 to 31 loop
    	if ((wk_number mod 2) = 0) then
	    wk_bv(i) := '0';
	else
	    wk_bv(i) := '1';
	end if;
	wk_number := wk_number / 2;
    end loop;

    if in_number < 0 then
    	i := 0;
	while ((i <= 31) and (wk_bv(i) = '0')) loop
	    i := i + 1;
	end loop;
	i := i + 1;
	for j in 31 downto i loop
	    wk_bv(j) := invert_bit(wk_bv(j));
	end loop;
    end if;
    return wk_bv;

end bitvectorof;

--write string to stdout with newline.

procedure print_formatted_message (message : string) is

    variable message_line : line;

begin

    write(message_line, message);
    writeline(std.textio.output, message_line);

end print_formatted_message;


--write debug message to stdout with newline.

procedure print_debug (name : string;
                       debug_control : debug;
		       message : string) is

    function everything_info (debug_control : debug) return string is
    	begin
	    if debug_control = everything then
	    	return "At (" & stringof(now) & ").  ";
	    else
	    	return "";
	    end if;
    end everything_info;

    function print_name (name : string) return string is
    	begin
	    if name /= "" then
	    	return name & ", ";
	    else
	    	return "";
	    end if;
    end print_name;
    
begin

    print_formatted_message(print_name(name) &
			    everything_info(debug_control) &
			    message);

end print_debug;


--generic resolution function for signals with only one driver at any
--time.  

function only_one_bv (bv : array_of_bvs) return bit_vector is

variable resolved_bv : bit_vector (31 downto 0);

begin

    resolved_bv := (others => '0');

    for i in bv'range loop
        resolved_bv := resolved_bv or bv(i);
    end loop;

    return resolved_bv;

end only_one_bv;


--generic resolution function for signals with only one driver at any
--time.  

function only_one_b (b : array_of_bs) return bit is

variable resolved_bit : bit;

begin

    resolved_bit := '0';

    for i in b'range loop
        resolved_bit := resolved_bit or b(i);
    end loop;

    return resolved_bit;

end only_one_b;

function my_res (b : array_of_bs) return bit is
variable resolved_bit : bit;

begin
  
   resolved_bit := '1';
   for i in b'range loop
	if b(i) /= '1' then
	    resolved_bit := '0';
	end if;
   end loop;

   return resolved_bit;

end my_res;

end compose;

